// Genesis.m
//--------------------------------------------------------------------

#import "Genesis.h"
#import <simtools.h>
#import "Macro.h"
@implementation Genesis

- setEvolutionFrequency: (int)o
{
  evolutionFrequency=o;
  return self;
}


- setMutationProbability: (float)o
{
  mutationProb=o;
  return self;
}

- setPopulationList: pL
{

   populationList = pL;
   return self;

}

- setSelectionPressure: (float)o
{
  selectionPressure=o;
  return self;

}


//====================================================================
//  Part 2 - createEnd
//====================================================================

//--------------------------------------------------------------------
//Here we build lists to handle evolution and set to zero statistics
//counters

- createEnd
{

   id obj;
   obj = [super createEnd];
   return obj;

}

//====================================================================
//  Part   - core: main-line and service-methods
//====================================================================

//--------------------------------------------------------------------
//This part of code contains the main-line of the process, single
//function will be performed by specific methods

- step
{


  //     [populationList forEach: M(print)];

     [self keepData];
     [self selectProportional];
     [self mutate];
     [self shuffle: populationList];




   return self;

}

//--------------------------------------------------------------------
//Here we verify operational condition about evolution frequency
//if evolutionFrequency == 1 evolution is done everytime
//
//if evolutionFrequency >  1 evolution is done every evolutionFrequency
//                                             times
//if evolutionFrequency is between 0 and 1 evolution is done if a
//   random real number between 0 and 1 is less than evolutionFrequency
//
//if evolutionFrequency == 0 no evolutions are done at all
//


- keepData
{

   float s;
   int i;
   id anInterface;


   //The number of individuals in the list is computed all times,
   //this way, users can remove or add several individuals between
   //two evolutions.


   numberOfIndividuals = [populationList getCount];
   numberOfFits=(int) numberOfIndividuals*(1-selectionPressure);
   if (numberOfFits <1) numberOfFits=1;
   maxFitness=-9999999.0;
   minFitness=99999999.0;

   for(i=0;i<numberOfIndividuals;i++)
   {

      anInterface   = (Person *) [populationList atOffset: i];
      s             = [anInterface getFitness];
      if ((maxFitness < s)) maxFitness = s;
      if ((minFitness > s)) minFitness = s;
   }


      return self;

}

//--------------------------------------------------------------------
//Here we select parents using a roulette-weel algorithm based upon
//fitness value


- selectSome
{
  int i;
 [QSort sortObjectsIn: populationList];
 while ([populationList getCount] > numberOfFits ) {
 [[populationList remove: [populationList atOffset:0]] drop];
 }

 [QSort reverseOrderOf: populationList];
  for (i=0;i<5;i++) {
 [ [populationList atOffset: i] print];
 }
 printf("\n");
 return self;

 [self reproduce];
  return self;
}

- selectProportional
{

  id tgt;
  id new;
  id old;
  id index;
  float q;

  q=maxFitness-((maxFitness-minFitness)*selectionPressure);
  if (q < minFitness) printf("Warning: there's too little selection pressure\n");

  old=[List create: [self getZone]];
  index=[populationList begin:[self getZone]];
  while ((tgt=[index next]))    [old addLast:tgt];
  [index drop];

  [populationList removeAll];

  index=[old begin:[self getZone]];
  while ([populationList getCount]< numberOfIndividuals) {

    if (!(tgt=[index next])) { [index setLoc: Start]; tgt=[index next];}


     if ([tgt getFitness] >= randbl(minFitness,q) ) {
       new =[tgt copy: [self getZone]];
       [populationList addLast: new];
    }
  }
  [index drop];
  [old forEach: M(drop)];
  [old drop];
  [populationList forEach: M(reset)];

  return self;
}





- reproduce {
  int i;
  id aMember;
  id  newMember;

   for (i=0;i<(numberOfIndividuals-numberOfFits);i++) {
   aMember=[populationList atOffset:i];
   newMember =[aMember copy: [self getZone]];
   [populationList addLast: newMember];
  }
   [populationList forEach: M(reset)];
  
  return self;

}



- mutate {
  int  i;
  for (i=0;i<numberOfIndividuals;i++) {
  if prob(mutationProb) {
  [[populationList atOffset: i] mutate]; 
 
  }

} 
  
  return self;
}

//--------------------------------------------------------------------

//Here we shuffle a list

- shuffle: (id) list
{

   int i,num;

   num = [list getCount];

   for(i=0;i<num;i++)
   [list addLast: [list remove : [list atOffset:
   [uniformIntRand getIntegerWithMin: 0 withMax: num-1]]]];
   return self;

}

//--------------------------------------------------------------------
@end
