// Hello, World! application. Copyright  1996-2000 Swarm Development Group.
// This library is distributed without any warranty; without even the
// implied warranty of merchantability or fitness for a particular purpose.
// See file LICENSE for details and terms of copying.

#import "ModelSwarm.h"
#import <simtools/QSort.h>
#import <random.h>
#include "Macro.h"
#import <analysis.h>


// Implementation of the model for ppl.

@implementation ModelSwarm






+ createBegin: aZone
{
  ModelSwarm *obj;
  id <ProbeMap> probeMap;

  obj = [super createBegin: aZone];

// chunk of PplModelSwarm
// define model-specific parameters and their probes
// This is included in ModelSwarm.m

  obj->increment = 1;
  obj->numPpl = 100;
  obj->round = 20;
  obj->mutation=0.01;
  obj->selectPressure=0.000;



  // MODIFY: add in defaults for your simulation parameters.
   probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
   probeMap = [probeMap createEnd];

  // Add probe to tell how often to increment the counter
  [probeMap addProbe: [probeLibrary getProbeForVariable: "increment"
				    inClass: [self class]]];
  // add probe to set number of people
  [probeMap addProbe: [probeLibrary getProbeForVariable: "numPpl"
                                    inClass: [self class]]];


  [probeMap addProbe: [probeLibrary getProbeForVariable: "round"
                                    inClass: [self class]]];

  [probeMap addProbe: [probeLibrary getProbeForVariable: "mutation"
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "selectPressure"
                                    inClass: [self class]]];


  [probeMap addProbe: [probeLibrary getProbeForVariable: "assortment"
                                    inClass: [self class]]];



  // install probemap into the library
  [probeLibrary setProbeMap: probeMap For: [self class]];



  
  return obj;
}

// MODIFY: possible creation objects here.
- createEnd
{
 return [super createEnd];
}

- buildObjects
{
  int inci;


  [super buildObjects];



   counter = [Counter create: [self getZone]];
   [counter setIncr: increment];
   [counter storeAValue: 20];

   
  
  // build the list to keep track of the ppl
   pplList = [List create: [self getZone]];

  for (inci = 0; inci < numPpl; inci++)
    {
// read the chunk BuildTheAgents.m for code to build the agents
#include "BuildOneAgent.m"      

    }

  // Creates the EZDistribution

  calc=[EZDistribution createBegin: [self getZone]];

  [calc setTitle: "stats"];
  [calc setGraphics:0];
  [calc setBinCount: 5];
  [calc setLowerBound: 0];
  [calc setUpperBound: 5];
  [calc setCollection: pplList ];
  [calc setProbedSelector: M(getTypology) ];
  [calc createEnd];

  // end EZDistribution

   genesis= [Genesis createBegin: self];
  
   [genesis setEvolutionFrequency: round];
   [genesis setPopulationList: pplList];
   [genesis setMutationProbability: mutation];
   [genesis setSelectionPressure: selectPressure];
   genesis=[genesis createEnd];


   groups= [Groups createBegin: self];
   [groups setNumberOfGroups: 2];
   [groups setPopulation: pplList];
   [groups setFunction: M(getProfile)];
   groups=[groups createEnd];
   [groups setName: "Plain"  group: 0];
   [groups setName: "Imita"  group: 1];


   groups2= [Groups createBegin: self];
   [groups2 setNumberOfGroups: [game getNActions]];
   [groups2 setPopulation: pplList];
   [groups2 setFunction: M(getTypology)];
   groups2=[groups2 createEnd];









 

  return self;
}

- buildActions
{
  [super buildActions];
  
  // create an action group to delineate the events
  modelActions = [ActionGroup create: [self getZone]];
  // Create an action to send a message to the counter every cycle
  //    telling it to execute "step"

  [modelActions createActionTo: groups2 message: M(step)];
  [modelActions createActionForEach: pplList message: M(step)];


  evolutionActions = [ActionGroup create: [self getZone]];
  [evolutionActions createActionTo: counter message: M(reset)];
  [evolutionActions createActionTo: genesis message: M(step)];
  [evolutionActions createActionTo: groups message: M(step)];




  // Create an action to send a message to each member of pplList
  //    every cycle telling them to execute "step"


  // Create a schedule for the events in modelActions
  modelSchedule = [Schedule createBegin: [self getZone]];
  // tell the model schedule to do the same thing at each time step
  [modelSchedule setRepeatInterval: 1];
  // re-allocate the modelSchedule and finish create phase
  modelSchedule = [modelSchedule createEnd];
  // tell the schedule to start doing the events in modelActions 
  //    at time 0  (the setRepeatInterval ensures that it happens
  //    again at time 1, 2, 3, ...)
  [modelSchedule at: 0 createAction: modelActions];


  evolutionSchedule = [Schedule createBegin: [self getZone]];
  // tell the model schedule to do the same thing at each time step
  [evolutionSchedule setRepeatInterval: round];
  // re-allocate the modelSchedule and finish create phase
  evolutionSchedule = [evolutionSchedule createEnd];
  // tell the schedule to start doing the events in modelActions 
  //    at time 0  (the setRepeatInterval ensures that it happens
  //    again at time 1, 2, 3, ...)
  [evolutionSchedule at: 0 createAction: evolutionActions];


  return self;
}

- activateIn: swarmContext
{

  // send a message to our super activating ourselves in their schedule
  [super activateIn: swarmContext];
  // then send a message to our schedule to start our actions
  [modelSchedule activateIn: self];
  [evolutionSchedule activateIn: self];
  // pass back the SwarmActitivity in the queue
  return [self getSwarmActivity];
}

// the objects will call this when they're done talking


// here we give the EZDistribution to whoever wants it.


- getCalc {

  return calc;

}




- getGame
{
  return game;
}

- setGame: g
{
  game=g;
  return self;
}

-getGroups 
{
  return groups;
}

-getGroups2 
{
  return groups2;
}


-getPopulation 
{
  return pplList;

}

- (Counter *)getCounter
{
  return counter;
}


-(float) getParam1 {
 return  assortment;
}

-(float) getParam2 {
 return  0.0;
}

@end
