// Hello, World! aication. Copyright  1996-2000 Swarm Development Group.
// This library is distributed without any warranty; without even the
// implied warranty of merchantability or fitness for a particular purpose.
// See file LICENSE for details and terms of copying.

#import "ObserverSwarm.h"
#import <analysis.h> // ActiveGraph
#import <gui.h>
#import "Game.h"
#import "Groups.h"

// implementation of sample graphical interface for models.
@implementation ObserverSwarm

+ createBegin: aZone
{
  ObserverSwarm * obj;
  id <ProbeMap> probeMap;
  
  obj = [super createBegin: aZone];

  // MODIFY: initialize other display options here.
  obj->displayFrequency = 1;
  obj->logIn = 0;

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  // MODIFY: set any display parameters here.
  [probeMap addProbe: [probeLibrary getProbeForVariable: "displayFrequency"
				    inClass: [self class]]];

  [probeMap addProbe: [probeLibrary getProbeForVariable: "logIn"
				    inClass: [self class]]];


  [probeLibrary setProbeMap: probeMap For: [self class]];

  return obj;
}

// MODIFY: possible creation finalization here
- createEnd
{
  
  return [super createEnd];
}

- buildObjects
{

 int inci;
 id modelZone;
 id game;
 id groups;
 BOOL graph;

 graph=YES;

  [super buildObjects];
  
  // allocate space to hold the model


  if ((game = [lispAppArchiver getWithZone: self 
                                            key: "game"]) == nil)
    {
  game = [Game createBegin: [self getZone]];
  game =[game createEnd];

    }

  if ((modelSwarm = [lispAppArchiver getWithZone: self 
                                            key: "modelSwarm"]) == nil)
    {
  modelZone = [Zone create: [self getZone]];
  modelSwarm = [ModelSwarm create: modelZone];
    }




  
  // create the probedisplay manager for the model swarm
  CREATE_PROBE_DISPLAY (modelSwarm);
  CREATE_PROBE_DISPLAY (game);
  // create the probe display for the observer swarm (self)
  CREATE_PROBE_DISPLAY (self);

  // set the control panel state
  [actionCache waitForControlEvent];
  // test to see if the "quit" button was hit
  if ([controlPanel getState] == ControlStateQuit)
    return self;

  // build the objects defined in modelSwarm.m
  [modelSwarm setGame: game];
  [modelSwarm buildObjects];
        
  groups=[modelSwarm getGroups];
  if (logIn == 1)   [groups toFile:"pop.dat"];
  // display the counter
  observedCounter = [modelSwarm getCounter];


  popGraph = [EZGraph createBegin: [self getZone]];
  [popGraph setTitle: "population"];
  [popGraph setAxisLabelsX: "Cycle" Y: "% of agents "];
  popGraph = [popGraph createEnd];


    for (inci=0;inci<[groups getCount] ;inci++) {

   [popGraph createSequence: (char *) [groups getName:inci]
		  withFeedFrom: [groups getGroup: inci]
		  andSelector: M(getCount)];

    }

#include "aGraph.ch.m"
#include "cGraph.ch.m"



  return self;
}  

- buildActions
{
  // build the actions for the observer's schedule
  [super buildActions];
  
  // build the actions for the model's schedule
  [modelSwarm buildActions];

  // create the display actions
  displayActions = [ActionGroup create: [self getZone]];
  // update the counter
  //  [displayActions createActionTo: observedCounter     message: M(print)];
  // update the graph

   [displayActions createActionTo: popGraph      message: M(step)];
   [displayActions createActionTo: aGraph      message: M(step)];
   [displayActions createActionTo: bGraph      message: M(step)];
   // check to see if everybody's done talking
  if (logIn ==1)   [displayActions createActionTo: [modelSwarm getGroups]    message: M(printGroups)];
  // update the probes
  // [displayActions createActionTo: probeDisplayManager message: M(update)];
  // check the controlpanel for clicks
  [displayActions createActionTo: actionCache         message: M(doTkEvents)];

  // then actually schedule the display actions
  displaySchedule = [Schedule createBegin: [self getZone]];
  [displaySchedule setRepeatInterval: [[modelSwarm getCounter] getAValue]];
  displaySchedule = [displaySchedule createEnd];
  [displaySchedule at: 1 createAction: displayActions];

  return self;
}  

- activateIn: swarmContext
{
  // put self (observer swarm) on the schedule of the GUI swarm
  [super activateIn: swarmContext];
  
  // start the activity for the model swarm
  [modelSwarm activateIn: self];
  // start the display schedule
  [displaySchedule activateIn: self];
  
  return [self getSwarmActivity];
}

// this method queries the ModelSwarm to find out if everybody
// has sent the ImFinished message to the modelswarm.  If so,
// then stop the sim because nobody's talking.





@end
